#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <my_global.h>
#include <my_sys.h>
#include <m_string.h>
#include "mysql.h"

#define HOST "127.0.0.1"
#define USERNAME "root"
#define PASSWORD ""
#define DATABASE "test"
#define QUERY "SELECT ? AS test_string"
#define PARAM "non-empty-string"
#define RESULT_BUFFER_LEN 128
//ME
#define MAX_RES_FIELDS 50
#define MAX_FIELD_DATA_SIZE 255

int main(int argc, char *argv[])
{
  MYSQL *link = NULL;
  MYSQL_STMT *ps = NULL;
  MYSQL_BIND params[1];
  MYSQL_BIND results[1];
  //REM by ME
  //unsigned long param_length;
  //unsigned long result_length;
  char *result_string = NULL;
  //ME
  int         field_count;
  MYSQL_RES   *result;
  char        data[MAX_RES_FIELDS][MAX_FIELD_DATA_SIZE];
  ulong       param_length[MAX_RES_FIELDS];
  my_bool     is_null[MAX_RES_FIELDS];
  int         row_count, i;
  ulong       result_length[MAX_RES_FIELDS];


  printf("status: Init\n");
  link = mysql_init(NULL);

  printf("status: Connect\n");
  mysql_real_connect(link, HOST, USERNAME, PASSWORD, DATABASE, 0, NULL, CLIENT_FOUND_ROWS|CLIENT_MULTI_STATEMENTS);
  
  if(mysql_errno(link)) goto error;

  printf("status: init prepared statement\n");
  ps = mysql_stmt_init(link);
  if(mysql_errno(link)) goto error;

  printf("status: prepare statement\n");
  mysql_stmt_prepare(ps, QUERY, strlen(QUERY));
  if(mysql_errno(link)) goto error;

  printf("status: bind parameters\n");
  memset(params, 0, sizeof(params));
  params[0].buffer_type = MYSQL_TYPE_VAR_STRING;
  params[0].buffer = PARAM;
  //ME
  params[0].buffer_length = strlen(PARAM);
  param_length[0] = RESULT_BUFFER_LEN * sizeof(char);//strlen(PARAM);
  params[0].length = &param_length[0];

  printf("status: set param string to '%s'\n", (char*)params[0].buffer);
  mysql_stmt_bind_param(ps, params);
  if(mysql_errno(link)) goto error;

  printf("status: executing statement\n");
  mysql_stmt_execute(ps);
  if(mysql_errno(link)) goto error;

  printf("status: binding result to buffers\n");
  //result_string = (char*)calloc(RESULT_BUFFER_LEN, sizeof(char));
  //memset(results, 0, sizeof(results));
  //results[0].buffer_type = MYSQL_TYPE_VAR_STRING;
  //results[0].buffer = result_string;
  //results[0].buffer_length = RESULT_BUFFER_LEN * sizeof(char);
  //results[0].length = &result_length;


  //ME
  if (!(result= mysql_stmt_result_metadata(ps))) /* No meta info */
  {
    while (!mysql_stmt_fetch(ps))
      row_count++;
    //return row_count;
  }

  field_count= min(mysql_num_fields(result), MAX_RES_FIELDS);

  bzero((char*) results, sizeof(results));
  bzero((char*) result_length, sizeof(result_length));
  bzero((char*) is_null, sizeof(is_null));

  for(i= 0; i < field_count; i++)
  {
    results[i].buffer_type= MYSQL_TYPE_STRING;
    results[i].buffer_length= MAX_FIELD_DATA_SIZE;//sizeof(result_length);//
    results[i].length= &result_length[i];
    results[i].buffer= (void *) data[i];
    results[i].is_null= &is_null[i];
  }



  mysql_stmt_bind_result(ps, results);
  if(mysql_errno(link)) goto error;

  //ME
  printf("status: store result\n");
  mysql_stmt_store_result(ps);

  printf("status: fetch row\n");
  mysql_stmt_fetch(ps);
  if(mysql_errno(link)) goto error;

  printf("RESULTS\n");
  printf("Expected one result ('" PARAM "'). Got:\n");
  printf("'%s'\n", (char*)results[0].buffer);
  //printf("'%s'\n", (char*)data[0]);

 cleanup:
  if(NULL != ps) {
    printf("status: close statement\n");
    mysql_stmt_close(ps);
    if(mysql_errno(link)) goto error;
  }
  
  printf("status: Disconnect\n");
  mysql_close(link);

  return 0;

 error:
  printf("ERROR: %s\n", mysql_error(link));

  if(NULL != ps) {
    mysql_stmt_close(ps);
    if(mysql_errno(link)) {
      printf("Error closing statement: %s\n", mysql_error(link));
    }
  }

  if(NULL != result_string) {
    free(result_string);
  }

  mysql_close(link);

  return 1;
}
